#include <windows.h>
#include <stdio.h>
#include <cstdio>
#include <iostream>
#include "Orchestrator_JNI.h"
#include "SendKeys.h"

HWND g_hwnd = 0;
/**
*	This function is called by EnumWindows, it loop on every windows, provides their handle and arg from enumwindows()
*	returns TRUE if still need to loop on windows, FALSE if not
**/

BOOL CALLBACK enumWindowsProc(HWND hWnd,LPARAM lParam) {
  int length = GetWindowTextLength( hWnd );
  if( 0 == length ) return TRUE;

  TCHAR* buffer;
  buffer = new TCHAR[ length + 1 ];
  memset( buffer, 0, ( length + 1 ) * sizeof( TCHAR ) );

  GetWindowText( hWnd, buffer, length + 1 );
  std::string windowTitle = std::string( buffer );
  delete[] buffer;
  
  // std::cout << "windowTitle=[" << windowTitle << "] lParam=[" << (std::string) lParam << "]" << std::endl;;
   if(windowTitle == (std::string) lParam)
   {
	   //std::cout << "windowTitle=[" << windowTitle << "] lParam=[" << (std::string) lParam << "] hwnd=[" << hWnd << "]" << std::endl;
	   g_hwnd = hWnd;
	   return false;
   }
   else
   {
	   return true;
   }
  
}


/**
*	This Function will set the targeted window title bar by default_name 
*   it returns true if everything went ok, -1 if not
**/
JNIEXPORT jboolean JNICALL Java_Orchestrator_JNI_setWindowName (JNIEnv * env , jobject obj, jstring default_name, jint hwnd)
  {
	  const char *def_name = (*env).GetStringUTFChars(default_name, NULL);
	  const char *ch_def_name = def_name;
	  
	  (*env).ReleaseStringUTFChars(default_name, def_name);

	  if(SetWindowText(hwnd, ch_def_name))
	  {  
		return true;
	  }
	  else
	  {
		return -1;  
	  }
	  
  }
  
  /**
  *		This Function will return true if character_name equals the title bar of the targeted window
  *		false if not
  **/
  
  JNIEXPORT jboolean JNICALL Java_Orchestrator_JNI_same (JNIEnv * env , jobject obj, jint hwnd, jstring character_name)
  {
	int len = ::GetWindowTextLength( hwnd );
    char out[len+len];
    GetWindowText((HWND) hwnd, out, len+len);
	// std::cout << "getn() result=["<< out << "]" << std::endl;	
	const jclass stringClass = env->GetObjectClass(character_name);
    const jmethodID getBytes = env->GetMethodID(stringClass, "getBytes", "(Ljava/lang/String;)[B");
    const jstring charsetName = env->NewStringUTF("windows-1252");
    const jbyteArray stringJbytes = (jbyteArray) env->CallObjectMethod(character_name, getBytes, charsetName);
    env->DeleteLocalRef(charsetName);
    const jsize length = env->GetArrayLength(stringJbytes);
    const jbyte* pBytes = env->GetByteArrayElements(stringJbytes, NULL); 
		
	char* data= malloc(length+1);
	memcpy( data, pBytes, length);
	data[length+1] = '\0';

	env->ReleaseByteArrayElements(stringJbytes, pBytes, JNI_ABORT); 
    env->DeleteLocalRef(stringJbytes);	
	
	//std::cout << "out:[" << out << "] data:[" << data << "]" << std::endl;
	
	//now compare data and out
	int counter =0;
	
	
		for (counter;counter <= length-1 ;counter++)
		{
			//std::cout << "out:[" << out[counter] << "] data:[" << data[counter] << "]" << std::endl;
			if(data[counter] != out[counter] )
			{
				return false;
			}
		}
	return true; 	
  }
  
  /**
  *		This Function returns hwnd of the current foreground window
  *		In window giberrish language, the foreground wind is the one which is currently set to get input
  *		usually, this is the one which is in front of all others windows as windos OS is not designed to send keystrokes to a window in the background
  *     Technically it's possible, but does not works with every applications
  **/
JNIEXPORT jint JNICALL Java_Orchestrator_JNI_getforegroundhwnd(JNIEnv * env, jobject obj)
  {
	return  GetForegroundWindow();
  }
  
  /**
  *		This Function returns the hwnd of the window have the titlebar default_name
  **/
JNIEXPORT jint JNICALL Java_Orchestrator_JNI_gethwnd(JNIEnv * env, jobject obj, jstring default_name)
  {	

	const jclass stringClass = env->GetObjectClass(default_name);
    const jmethodID getBytes = env->GetMethodID(stringClass, "getBytes", "(Ljava/lang/String;)[B");
    const jstring charsetName = env->NewStringUTF("windows-1252");
    const jbyteArray stringJbytes = (jbyteArray) env->CallObjectMethod(default_name, getBytes, charsetName);
    env->DeleteLocalRef(charsetName);
    const jsize length = env->GetArrayLength(stringJbytes);
    const jbyte* pBytes = env->GetByteArrayElements(stringJbytes, NULL); 
		
	char* data= malloc(length+1);
	memcpy( data, pBytes, length);
	data[length] = '\0';

	env->ReleaseByteArrayElements(stringJbytes, pBytes, JNI_ABORT); 
    env->DeleteLocalRef(stringJbytes);		 
	//std::cout << "gethwnd() data=[" << data << "]" <<'\n';
	EnumWindows(enumWindowsProc, data);
	
 	//std::cout << "Enum Desktop Windows" << '\n';
	
	HWND hwnd;
    hwnd = g_hwnd;
		
	if ( hwnd != 0 ) 
	{
		return hwnd;
	}
	else
	{
		return -1;
	}	
}
  
/**
*	This Function will input the string command in the targeted window
**/
JNIEXPORT void JNICALL Java_Orchestrator_JNI_sync (JNIEnv * env, jobject obj, jstring command, jint handle)
  {
	const jclass stringClass = env->GetObjectClass(command);
    const jmethodID getBytes = env->GetMethodID(stringClass, "getBytes", "(Ljava/lang/String;)[B");
    const jstring charsetName = env->NewStringUTF("windows-1252");
    const jbyteArray stringJbytes = (jbyteArray) env->CallObjectMethod(command, getBytes, charsetName);
    env->DeleteLocalRef(charsetName);
    const jsize length = env->GetArrayLength(stringJbytes);
    const jbyte* pBytes = env->GetByteArrayElements(stringJbytes, NULL); 
	
	//this make sure our string is C/C++ compliant with null character
	char* g_command = malloc(length+1);
	memcpy( g_command, pBytes, length);
	g_command[length] = '\0';

	env->ReleaseByteArrayElements(stringJbytes, pBytes, JNI_ABORT); 
    env->DeleteLocalRef(stringJbytes);	
	
	CSendKeys sk;

	sk.AppActivate((HWND) handle);
	
	
	sk.SendKeys("?");
	sk.SendKeys(g_command);
	sk.SendKeys("?");
  }
  
  /**
  *		This Function minimize or maximize the targeted window based on swcode
  **/
  JNIEXPORT void JNICALL Java_Orchestrator_JNI_minimize (JNIEnv * env, jobject obj, jint hwnd, jint sw_code)
  {
	  ShowWindow((HWND) hwnd, sw_code);	  
  }
  
  /**
  *		This Function is supposed to tell you whether or not if the targeted windows is still alive
  **/
  JNIEXPORT jboolean JNICALL Java_Orchestrator_JNI_iswindow (JNIEnv * env, jobject obj, jint hwnd)
  {
	  return IsWindow( (HWND) hwnd) & IsWindowVisible( (HWND) hwnd );
  }
  
  
  JNIEXPORT void JNICALL Java_Orchestrator_JNI_sendkey (JNIEnv *env, jobject obj, jint keycode, jint hwnd)
  {
	if(SetForegroundWindow( (HWND) hwnd) != 0 )
	{
	//std::cout << "windows successfully set to foreground" << std::endl;
	//std::cout << "map VK " <<  keycode << " to scan code " << MapVirtualKey(keycode,0);
	keybd_event(keycode,MapVirtualKey(keycode,0),0,0);
	Sleep(40);
	keybd_event(keycode,MapVirtualKey(keycode,0),2,0);
	Sleep(40);
	/**
	* F1 = 3B
	* F2 = 3C
	* F3 = 3D
 	* F4 = 3E
	* F5 = 3F
	*
	**/
	}
  }
  
  
  JNIEXPORT jboolean JNICALL Java_Orchestrator_JNI_setforeground (JNIEnv * env, jobject obj, jint hwnd)
  {
	  if(SetForegroundWindow( (HWND) hwnd) != 0 )
    {
		return true;
	}		
	else
	{
		return false;
	}
  }
  
  
  
  
  
  
  
  
  